import os.path
import re
from lxml import etree

from granulite import settings


class TransformationMiddleware(object):
    
    def process_request(self, request):
        """
        Checks if the request contains a special GET parameter to transform
        the view. If so, it is stored for use in the response phase and
        stripped from the request.
        """
        stylesheet_key = settings.TRANSFORM_STYLESHEET_KEY
        get_params = request.GET.copy()
        if stylesheet_key in request.GET:
            self.xsl_filename = get_params[stylesheet_key]
            del get_params[stylesheet_key]
        else:
            self.xsl_filename = None # Important to reset each request

        params_key = settings.TRANSFORM_PARAMS_KEY
        if params_key in request.GET:
            self.xsl_params_string = get_params[params_key]
            del get_params[params_key]
        else:
            self.xsl_params_string = None
        request.GET = get_params

    def process_response(self, request, response):
        """
        Attempts to transform the response content with the XSL filename that
        was stored in the request phase. 
        """
        if self.xsl_filename is not None and response.status_code == 200:
            views_dir = settings.TRANSFORM_XSL_DIR
            xsl_file_path = os.path.join(views_dir, '%s.xsl' % self.xsl_filename)
            xsl_doc = etree.parse(xsl_file_path)
            transform = etree.XSLT(xsl_doc)
            doc = etree.fromstring(response.content)
            response.content = transform(doc, **self.xsl_params)
        return response
    
    @property
    def xsl_params(self):
        """
        Returns a dict of XSLT parameters as extracted from the GET parameter
        stored in the request phase.
        
        For instance, if the GET parameter was ``[p1='foo',p2='bar',p3=77]``, this would result in the following dict:
        {
            'p1': "'foo'",
            'p2': "'bar'",
            'p3': "77",
        }
        Not that everything is interpreted as string by default, to allow for
        the difference between XPath expression and true string values.
        """
        xsl_params = {}
        pipe_separators = True
        if self.xsl_params_string is not None:
            params_string = self.xsl_params_string
            
            # TODO implement more elegant mechanism for allowing custom parsing of XSL parameters?
            if pipe_separators:
                matches = re.findall(r'(.+?)\*\|\*(.+?)\^\|\^', params_string)
                for param in matches:
                    xsl_params[str(param[0])] = param[1]
            else:
                match = re.match(r'^\[(.+)\]$', params_string)
                if not match:
                    raise ValueError('Invalid format for XSL parameters.')
                params = match.group(1).split(',')
                for param in params:
                    param_name, param_value = param.split('=')
                    xsl_params[str(param_name).strip()] = param_value
        return xsl_params
    
